@@ -1,12 +1,6 @@
 # frozen_string_literal: true
 
-require 'minitest/global_expectations/autorun'
-require 'rack'
-require 'rack/cascade'
-require 'rack/files'
-require 'rack/lint'
-require 'rack/urlmap'
-require 'rack/mock'
+require_relative 'helper'
 
 describe Rack::Cascade do
   def cascade(*args)
@@ -37,10 +31,42 @@ describe Rack::Cascade do
     Rack::MockRequest.new(cascade).get("/cgi/../bla").must_be :not_found?
   end
 
+  it "include? returns whether app is included" do
+    cascade = Rack::Cascade.new([app1, app2])
+    cascade.include?(app1).must_equal true
+    cascade.include?(app2).must_equal true
+    cascade.include?(app3).must_equal false
+  end
+
   it "return 404 if empty" do
     Rack::MockRequest.new(cascade([])).get('/').must_be :not_found?
   end
 
+  it "uses new response object if empty" do
+    app = Rack::Cascade.new([])
+    res = app.call('/')
+    s, h, body = res
+    s.must_equal 404
+    h['Content-Type'].must_equal 'text/plain'
+    body.must_be_empty
+
+    res[0] = 200
+    h['Content-Type'] = 'text/html'
+    body << "a"
+
+    res = app.call('/')
+    s, h, body = res
+    s.must_equal 404
+    h['Content-Type'].must_equal 'text/plain'
+    body.must_be_empty
+  end
+
+  it "returns final response if all responses are cascaded" do
+   app = Rack::Cascade.new([])
+   app << lambda { |env| [405, {}, []] }
+   app.call({})[0].must_equal 405
+  end
+
   it "append new app" do
     cascade = Rack::Cascade.new([], [404, 403])
     Rack::MockRequest.new(cascade).get('/').must_be :not_found?
